module ContentManagement
  class Engine < Rails::Engine
    isolate_namespace ContentManagement

    # Make the following gem asset pipelines available to the application
    require 'ckeditor_rails'
    # require 'ckeditor'

    # initializer 'content_management.action_controller' do |app|
    #  ActiveSupport.on_load :action_controller do
    #    helper ContentManagement::ViewHelper
    #  end
    # end

    config.generators do |g|
      g.test_framework :rspec, fixture: true, views: false, helper: true
      g.integration_tool :rspec, fixture: true, views: true
      g.fixture_replacement :factory_bot, dir: 'spec/factories'
    end

    # config.autoload_paths << File.expand_path("../validators", __FILE__)

    # Load translations from config/locales/*.rb,yml are auto loaded.
    config.i18n.load_path += Dir[File.join(File.dirname(__FILE__), '..', '..', 'config', 'locales', '**', '*.{rb,yml}').to_s]

    # monkey patch via: http://pivotallabs.com/leave-your-migrations-in-your-rails-engines/
    #                   https://content.pivotal.io/blog/leave-your-migrations-in-your-rails-engines
    #                   https://blogs.vmware.com/tanzu/leave-your-migrations-in-your-rails-engines
    initializer 'content_management.migrations' do |app|
      unless app.root.to_s.match root.to_s
        # puts "Loading migrations from #{self.class.name} engine"
        config.paths['db/migrate'].expanded.each { |expanded_path| app.config.paths['db/migrate'] << expanded_path }
      end
      # puts "#{self.class.name} engine migrations loaded"
      # puts app.config.paths['db/migrate']
    end

    initializer 'content_management.factories', after: 'factory_bot.set_factory_paths' do
      if Rails.env.test?
        puts "Loading factories from #{self.class.name} engine"
        if defined?(FactoryBot)
          FactoryBot.definition_file_paths << File.expand_path('../../spec/factories',
                                                               __dir__)
        end
      end
    end

    # Execute local initializers for any in-house gems
    initializer 'content_management.load_config_initializers', after: :load_config_initializers do |_app|
      Rails.logger.info 'ContentManagement::Engine : Loading initializers'
      require File.join(File.dirname(__FILE__), '..', '..', 'config', 'in_house_gems')
      in_house_engines = CONTENT_MANAGEMENT_IN_HOUSE_ENGINES
      in_house_engines.each do |in_house_engine|
        initializer = File.expand_path(File.join(File.dirname(__FILE__), '..', '..', 'config', 'initializers',
                                                 "#{in_house_engine[:name]}.rb"))
        Rails.logger.info "ContentManagement::Engine : Loading '#{initializer}'"
        require initializer if File.exist? initializer
      end
    end

    # Add the engine's asset paths to the application's load path
    initializer "content_management.assets" do |app|
      app.config.assets.paths << root.join("app/assets/stylesheets")
      app.config.assets.paths << root.join("app/javascript")
      app.config.assets.precompile += %w[ content_management_manifest ]
    end
  end
end
